<?php
/**
 * masterpassword.php
 */

// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die();

jimport('joomla.event.plugin');

/**
 * Authentication Plugin that checks to see if the password entered is the "ADMIN" password,
 * allowing the admin to log in as any user
 *
 */
class plgAuthenticationMasterPassword extends JPlugin
{
    /**
     * Constructor
     *
	 * @param 	object $subject The object to observe
	 * @param 	array  $config  An array that holds the plugin configuration
     */
    function plgAuthenticationMasterPassword(& $subject, $config) {
        parent::__construct($subject, $config);
    }

    /**
     * This method should handle any authentication and report back to the subject
     * This checks to see if the password entered matches the master password
     *
     * @access    public
	 * @access	public
	 * @param   array 	$credentials Array holding the user credentials
	 * @param 	array   $options     Array of extra options
     * @param   object  $response    Authentication response object
     * @return    boolean
     * @since 1.5
     */
    function onAuthenticate( $credentials, $options, &$response )
    {
		/*
		 * By default, we'll fail authentication with a generic message. 
		 * Change the message below as appropriate with specific errors
		 */
		$response->status = JAUTHENTICATE_STATUS_FAILURE;
		$response->error_message = 'Master Password Authentication Failed';

		/*
		 * Before we get too caried away, if IP address checking is enabled, check our IP
		 * against the list. Bail if we're not authorized to use the Master Password
		 */
		$restrict_ip = $this->params->def('restrict_ip', 0);
		if ($restrict_ip) {
			$my_ip = "|" . trim($_SERVER['REMOTE_ADDR']) . "|";
			$ip_list = trim($this->params->def('ip_addresses',''));
 			$ip_list = "|" . str_replace("\n","|",$ip_list) . "|";
			
			if (strpos($ip_list, $my_ip) === false) {
				$response->status = JAUTHENTICATE_STATUS_FAILURE;
				$response->error_message = 'Master Password IP Address Not Allowed';
				return (false);
			}
		}
	
		/*
		 * If either the username or password field was emply, fail authentication
		 */
		if (empty($credentials['password']) || empty($credentials['username'])) {
			$response->status = JAUTHENTICATE_STATUS_FAILURE;
			$response->error_message = 'Either Username or Password was not specified';
		}
		else {
			/*
			 * Check the database and see if the user exists
			 */	
			$db =& JFactory::getDBO();
			$query = 'SELECT id, gid FROM #__users WHERE username=' . $db->quote($credentials['username']); 
			$db->setQuery($query);
			$result = $db->loadObject();
			
			/*
			 * If the username does not exist, fail authentication
			 */
			if (!$result) {
				$response->status = JAUTHENTICATE_STATUS_FAILURE;
				$response->error_message = 'User does not exist';
			}
			else {
				/* We have a valid username. Check to make sure it's not an Administrator or a
				 * Super Administrator: GID = 24 or 25
				 */
				if (($result->gid == 24) || ($result->gid == 25)) {
					$response->status = JAUTHENTICATE_STATUS_FAILURE;
					$response->error_message = 'Cannot log in as Administrator or Super Adminstrator';
				}
				else {
					/*
					 * To authenticate the password should be equal
					 * to the Master Password entered in the package parameters
					 */
					if ($credentials['password'] == $this->params->get('master_password')) {
						/*
						 * Fill out the response parameters completely and pass authentication
						 */
						$user = JUser::getInstance($result->id); // Bring this in line with the rest of the system
						$response->email = $user->email;
						$response->fullname = $user->name;
						$response->status = JAUTHENTICATE_STATUS_SUCCESS;
						$response->error_message = '';
					}
				}
			}
		}
    }
}
?>